package eclipselink.cloning;

import static javax.persistence.CascadeType.ALL;
import static javax.persistence.FetchType.LAZY;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.OneToMany;

import org.eclipse.persistence.annotations.PrivateOwned;

@Entity
public class Department
{
	@Id
	@Column(name="DEPARTMENT_ID")
	private int id;
	
	@OneToMany(fetch=LAZY, mappedBy="department", cascade={ALL})
	@PrivateOwned
	private List<Employee> employees = new ArrayList<Employee>();
	
	@Column
	private String name;
	
	private void addEmployee(Employee toAdd)
	{
		getEmployees().add(toAdd);
		toAdd.setDepartment(this);
	}
	
	public List<Employee> getEmployees()
	{
		return employees;
	}
	
	public void setEmployees(List<Employee> newValue)
	{
		employees = newValue;
	}
	
	@Override
	public Object clone()
	throws CloneNotSupportedException
	{
		Department result = (Department) super.clone();
		
                result.setEmployees(new ArrayList<Employee>());
		
		for (Employee originalEmployee : this.getEmployees())
		{
			result.addEmployee((Employee) originalEmployee.clone());
		}
		
		return result;
	}
}
