package test;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import eclipselink.cloning.Department;
import eclipselink.cloning.Employee;

import junit.framework.TestCase;
import org.junit.Assert;
import org.junit.Test;

public class DepartmentCloneTest extends TestCase
{
        @Test
        public void testNothing()
        throws Exception
        {
        }

	@Test
	public void testDepartmentClone()
	throws Exception
	{
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("test-eclipselink-pu");
		EntityManager em = emf.createEntityManager();
		
		Query q = em.createQuery("select d from Department d");
		List<Department> departmentsRead = q.getResultList();
		
		Assert.assertNotNull("Unexpected number of Departments read", departmentsRead);
		Assert.assertEquals("Unexpected number of Departments read", 1, departmentsRead.size());
		Assert.assertNotNull("Read Department must be non-null", departmentsRead.get(0));
		
		Department department = departmentsRead.get(0);
		
		Assert.assertEquals("Unexpected number of Employees in original Department",
				            3,
				            department.getEmployees().size());
		
		for (Employee employee : department.getEmployees())
		{
			Assert.assertNotNull("Read Employee must be non-null",employee);
			Assert.assertNotNull("Read Employee name must be non-null and non-empty",
					             employee.getName());
			Assert.assertTrue("Read Employee name must be non-null and non-empty",
		                      employee.getName().trim().length() > 1);
			Assert.assertSame("Read Employee's Department does not match read Department",
					          department,
					          employee.getDepartment());
		}
		
		em.close();
		emf.close();
		
		Set<Employee> readEmployees = new HashSet<Employee>(department.getEmployees());
		Department departmentClone = (Department) department.clone();
		Set<Employee> clonedEmployees = new HashSet<Employee>(departmentClone.getEmployees());
		
		Assert.assertNotSame("Unexpected Department reference check result",
				             department,
				             departmentClone);
		Assert.assertEquals("Unexpected number of cloned Employees",
				            readEmployees.size(),
				            clonedEmployees.size());
		
		for (Employee employee : department.getEmployees())
		{
			Assert.assertSame("Incorrect back pointer from source Employee to source Department",
					          department,
					          employee.getDepartment());
			Assert.assertFalse("Cloned employees unexpectedly contains a source Employee",
					           clonedEmployees.contains(employee));
		}
		
		for (Employee employee : departmentClone.getEmployees())
		{
			Assert.assertSame("Incorrect back pointer from cloned Employee to cloned Department",
			                  department,
			                  employee.getDepartment());
			Assert.assertFalse("Source employees unexpectedly contains a cloned Employee",
			                   readEmployees.contains(employee));
		}
	}
}
